import fs from "fs"
import os from "os"
const commands = new Map()

export default async (client, properties = { directory: "commands", commandLogging: true }) => {
	for (let commandFile of fs.readdirSync(`./${properties.directory}`)) {
		import(`${os.type() == "Windows_NT" ? "../../" : process.cwd()}/${properties.directory}/${commandFile}`).then(File => {
			commands.set(File.default.name, File.default)
			if (properties.commandLogging) {
				console.log(`Synchronized command > ${File.default.name[0].toUpperCase() + File.default.name.slice(1)}`)
			}
		})
	}

	await (client.readyAt ? Promise.resolve() : new Promise(resolve => client.on('ready', resolve)))

	client.guilds.cache.forEach(async guild => {
		await guild.commands.fetch().then(() => {
			commands.forEach(Command => {
				if (Command.guild == guild.id) {
					guild.commands.create({
						type: Command.type,
						name: Command.name,
						description: Command.description,
						defaultMemberPermissions: Command.permissions,
						options: Command.options
					})
				} else {
					guild.commands.create({
						type: Command.type,
						name: Command.name,
						description: Command.description,
						defaultMemberPermissions: Command.permissions,
						options: Command.options
					})
				}
			})
	
			guild.commands.cache.forEach(Command => {
				if (!Array.from(commands.keys()).includes(Command.name)) {
					guild.commands.delete(Command.id)
				}
			})
		})
	})

	client.on('interactionCreate', async (interaction) => {
		if (interaction.isCommand()) {
			const command = commands.get(interaction.command.name)
	
			try {
				command.execute(client, interaction, interaction.options)
			} catch (error) {
				interaction.reply({
					embeds: [
						{
							title: '❌ Error ❌',
							description: 'An error occurred while running this command',
							color: 0xff0000
						}
					]
				})
				console.error(error)
			}
		} else if (interaction.isMessageComponent()) {
			const command = commands.get(interaction.customId.split(".")[0])
	
			try {
				command.componentInteraction(client, interaction, interaction.customId.split(".")[1])
			} catch (error) {
				console.error(error)
			}
		} else if (interaction.isAutocomplete()) {
			const command = commands.get(interaction.command.name)
	
			try {
				command.autocompleteInteraction(client, interaction, interaction.options.getFocused())
			} catch (error) {
				console.error(error)
			}
		} else if (interaction.isModalSubmit()) {
			const command = commands.get(interaction.customId.split(".")[0])
	
			try {
				command.modalInteraction(client, interaction, interaction.customId.split(".")[1])
			} catch (error) {
				console.error(error)
			}
		}
	})

	client.on('guildCreate', (guild) => {
		commands.forEach(command => {
			if (command.guild == guild.id) {
				guild.commands.create({
					type: command.type,
					name: command.name,
					description: command.description,
					defaultMemberPermissions: command.permissions,
					options: command.options,
				})
			} else {
				guild.commands.create({
					type: command.type,
					name: command.name,
					description: command.description,
					defaultMemberPermissions: command.permissions,
					options: command.options,
				})
			}
		})
	})
}